from pipeline import Screen
import time
import logging
moduleLogger = logging.getLogger(__name__)
class ExceptionHandler(Screen):
	def Execute(self, msg):
		try:
			self.target(msg)
		except Exception as e:
			msg.StopProcessing = True
			msg.Error = e
		return msg
		
class StopProcessingHandler(Screen):
	def Execute(self, msg):
		if msg.StopProcessing:
			return msg
		else:
			return self.target(msg)
			
			
class RetryScreen(Screen):
	def __init__(self, Retry = 3, slideTime = 5):
		self.Retry = Retry
		self.slideTime = slideTime
		self.currentRetry = 0
		Screen.__init__(self)
		
	def Execute(self, msg):
		logger = moduleLogger.getChild(self.__class__.__name__)
		msg = self.target(msg)
		try:
			if msg.Retry:
				self.currentRetry = self.currentRetry+1
				logger.debug('msg.Retry = True')
				if not self.currentRetry > self.Retry:
					logger.debug('Retrying %s of %s'%(str(self.currentRetry), str(self.Retry)))
					logger.debug('Sleeping %s'%str(self.slideTime*self.currentRetry))
					time.sleep(self.slideTime*self.currentRetry)
					msg.Retry = False
					msg = self.Execute(msg)
				else:
					logger.debug('Max Retries Reached')
					msg.StopProcessing = True
					msg.Error = 'Max retries reached'
			
		except AttributeError:
			pass
		self.currentRetry = 0
		return msg


import pytest

import pipewrench
import pipewrench.test.testobjs as testobjs
import logging
LOGLVL = logging.DEBUG

logging.basicConfig(level=LOGLVL, format='%(asctime)s %(name)s %(levelname)s: %(message)s', datefmt='%Y-%m-%d %H:%M:%S')

@pytest.fixture(scope = 'class')
def pipefitting():
	return pipewrench.PipeFitting()
	
@pytest.fixture()
def filter():
	return testobjs.TFilter()
	
@pytest.fixture()
def screen():
	return testobjs.TScreen()
	
@pytest.fixture()
def retryscreen():
	return pipewrench.RetryScreen(slideTime = 1)
	
@pytest.fixture()
def retryfilter():
	return testobjs.TFilterRetry()
	
class Test_PipeFitting(object):
	def test_addFilter(self, pipefitting, filter):
		pipefitting.addFilter(filter)
		
	def test_addFilter2(self, pipefitting, filter):
		pipefitting.addFilter(filter)
		
	def test_addFilter3(self, pipefitting, filter):
		pipefitting.addFilter(filter)
	
	def test_addFilterScreen(self, pipefitting, screen):
		assert pipefitting.addScreen(screen, True)
		
	def test_addGlobalScreen(self, pipefitting, screen):
		assert pipefitting.addScreen(screen)
		
	def test_Process(self, pipefitting):
		msg = pipefitting.Process(1)
		assert msg == 4
		
	def test_Step(self, pipefitting):
		for msg in pipefitting.Step(1):
			print msg
			
	
class Test_Retry(object):
	def test_addObjs(self, retryfilter, retryscreen, pipefitting):
		pipefitting.addFilter(retryfilter)
		pipefitting.addScreen(retryscreen, True)
		
	def test_Retry(self, pipefitting):
		msg = pipefitting.Process(0)
		assert msg == 4



import pipewrench.pipeline as pipeline

class TScreen(pipeline.Screen):
	def Execute(self, msg):
		msg = self.target(msg)
		if msg.StopProcessing:
			print 'FAIL'
		return msg
		
class TFilter(pipeline.Filter):
	def Execute(self, msg):
		msg = msg + 1
		if msg > 3:
			self.msg.StopProcessing = True
		return msg
		
		
class TFilterRetry(pipeline.Filter):
	def Execute(self, msg):
		msg = msg + 1
		if msg < 5:
			self.msg.Retry = True
		return msg


import logging

class Pipeline(object):
	def __init__(self):
		self.filters = []
		self.globalScreens = []
		self.filterScreens = {}
		
	def Execute(self, msg):
		for filter in self.filters:
			if msg.StopProcessing:
				return msg
			msg = self._processPipeline(filter, msg)
			
		return msg
		
	def addFilter(self, filter):
		self.filters.append(filter)
		
	def addScreen(self, screen, filter = None):
		#If filter is a filter instance the screen will be added to that filter
		#If filter is True the screen will be added to the last filter
		#If filter is omitted or None the screen will be applied globally
		
		if isinstance(filter, Filter):
			if filter in self.filterScreens:
				self.filterScreens[filter].append(screen)
			else:
				self.filterScreens[filter] = [screen]
		elif filter == None:
			self.globalScreens.append(screen)
			
		elif filter == True:
			if len(self.filters) > 0:
				f = self.filters[len(self.filters)-1]
				if f in self.filterScreens:
					self.filterScreens[f].append(screen)
				else:
					self.filterScreens[f] = [screen]
			else:
				return False
				
		return True
	
	def _processPipeline(self, filter, msg):
		return self._constructPipeline(filter)(msg)
		
		
	def _constructPipeline(self, filter):
		target = filter.Process
		if filter in self.filterScreens:
			for screen in self.filterScreens[filter]:
				screen.setTarget(target)
				target = screen.Execute
		for screen in self.globalScreens:
			screen.setTarget(target)
			target = screen.Execute
		
		return target
	
	def Step(self, msg):
		for filter in self.filters:
			msg = self._processPipeline(filter, msg)
			yield msg
	
class Screen(object):
	def __init__(self):
		self.target = None
		
	def setTarget(self, target):
		self.target = target
		
	def Execute(self, msg):
		return self.target(msg)
		
class Filter(object):
	def Process(self, msg):
		self.msg = msg
		if msg.Unpack:
			msg.payload = self.Execute(msg.payload)
		else:
			msg = self.Execute(msg)
		return msg
			
	def Execute(self, msg):
		return msg
		
	
		
class Message(object):
	def __init__(self, payload = None):
		self.StopProcessing = False
		self.Error = None
		self.Retry = False
		if not payload == None:
			self.payload = payload
			self.Unpack = True
		else:
			self.Unpack = False
			


from distutils.core import setup

setup(
    name='Pipewrench',
    version='0.0.2b',
	author='Taylor McKinnon',
    author_email='tokintmac@gmail.com',
    packages=['pipewrench', 'pipewrench.test'],
	url='https://github.com/TokinT-Mac/PipeWrench',
    license='GPLv3',
	description='Framework for pipeline-style message processing',
    long_description=open('README').read(),
)



import pipeline

class PipeFitting(object):
	def __init__(self):
		self.pipeline = pipeline.Pipeline()
		self.addFilter = self.pipeline.addFilter
		self.addScreen = self.pipeline.addScreen
		
	def Process(self, msg):
		msg = self.packMessage(msg)
		msg = self.pipeline.Execute(msg)
		msg = self.unpackMessage(msg)
		return msg
			
	def packMessage(self, msg):
		if isinstance(msg, pipeline.Message):
			return msg
		else:
			return pipeline.Message(msg)
			
	def unpackMessage(self, msg):
		if msg.Unpack:
			return msg.payload
		else:
			return msg
			
	def Step(self, msg):
		msg = self.packMessage(msg)
		for message in self.pipeline.Step(msg):
			msg = self.unpackMessage(message)
			yield msg